
/******************************************************************************
* MODULE     : android_system.cpp
* DESCRIPTION: Android system function proxies
* COPYRIGHT  : (C) 2024 Liza Belos
*******************************************************************************
* This software falls under the GNU general public license version 3 or later.
* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
******************************************************************************/

#include "android_system.hpp"
#include "config.h"

#include <QGuiApplication>
#include <QStyleHints>
#include <QFile>
#include <QString>
#include <QDir>
#include <QDirIterator>
#include <QIODevice>
#include <QTextStream>
#include <QDebug>
#include <QDateTime>

#include "../../Scheme/Guile/guile_tm.hpp"
#ifdef SCM_HAVE_HOOKS
#include "libguile/system.h"
#else
#error "SCM_HAVE_HOOKS is not defined"
#endif

inline QString texmacs_utf8_string_to_system_string (string utf8_string) {
  return QString::fromUtf8(&utf8_string[0], N(utf8_string));
}

inline string texmacs_ainsi_to_utf8(const QString &local_string) {
  return string(
    local_string.toUtf8().data(),
    (int)local_string.size()
  );
}

void texmacs_lock_file(FILE *&file) {
  // do nothing
}

void texmacs_unlock_file(FILE *&file) {
  // do nothing
}

extern QMap<QString, QString> *asset_files;

FILE* texmacs_fopen(string filename, string mode, bool lock) {
  QString qfilename = texmacs_utf8_string_to_system_string(filename);
  if (mode == "r" || mode == "rb") {
    if (asset_files->contains(qfilename)) {
      QFile *pfile = new QFile(asset_files->value(qfilename));
      if (pfile->open(QFile::ReadOnly)) {
        qDebug() << "Opening asset file " << qfilename;
        return (FILE*)pfile;
      }

    }
  }

  QFile *pfile = new QFile(qfilename);
  QFile::OpenMode open_mode = QFile::NotOpen;
  for (int i = 0; i < N(mode); i++) {
    switch (mode[i]) {
      case 'r':
        open_mode |= QFile::ReadOnly;
        break;
      case 'w':
        open_mode |= QFile::WriteOnly;
        break;
      case 'a':
        open_mode |= QFile::Append;
        break;
      case '+':
        open_mode |= QFile::ReadWrite;
        break;
      case 'b':
        // do nothing
        break;
      default:
        qDebug() << "Warning : invalid mode";
        break;
    }
  }
  if (!pfile->open(open_mode)) {
    return nullptr;
  }
  return (FILE*)pfile;
}

ssize_t texmacs_fwrite (const char *s, size_t n, FILE *stream) {
  if (stream == stdout) {
    qDebug() << QString::fromUtf8(s, n);
    return n;
  }
  if (stream == stderr) {
    qWarning() << QString::fromUtf8(s, n);
    return n;
  }
  QFile *pfile = (QFile*)stream;
  int written = pfile->write(s, n);
  if (written == -1) {
    return 0;
  }
  return written;
}

ssize_t texmacs_fsize (FILE *stream) {
  QFile *pfile = (QFile*)stream;
  return pfile->size();
}

ssize_t texmacs_fread (char *z, size_t n, FILE *stream) {
  QFile *pfile = (QFile*)stream;
  return pfile->read(z, n);
}

void texmacs_fclose(FILE *&file, bool unlock) {
  QFile *pfile = (QFile*)file;
  pfile->close();
  delete pfile;
  file = nullptr;
}

typedef struct texmacs_android_dir_t {
  QDirIterator *iterator;
  QString dirname;
} texmacs_android_dir_t;

TEXMACS_DIR texmacs_opendir(string dirname) {
  QDirIterator *iterator = new QDirIterator(
    texmacs_utf8_string_to_system_string(dirname),
    QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot,
    QDirIterator::Subdirectories
  );
  if (!iterator->hasNext()) {
    delete iterator;
    return nullptr;
  }
  texmacs_android_dir_t *dir = new texmacs_android_dir_t;
  dir->iterator = iterator;
  dir->dirname = texmacs_utf8_string_to_system_string(dirname);
  return (TEXMACS_DIR)dir;
}

void texmacs_closedir(TEXMACS_DIR dir) {
  texmacs_android_dir_t *dirp = (texmacs_android_dir_t*)dir;
  delete dirp->iterator;
  delete dirp;
}

texmacs_dirent texmacs_readdir(TEXMACS_DIR dirp) {
  texmacs_android_dir_t *dir = (texmacs_android_dir_t*)dirp;
  if (!dir->iterator->hasNext()) {
    return {false, ""};
  }
  return {true, texmacs_ainsi_to_utf8(dir->iterator->next())};
}

int texmacs_stat(string filename, struct stat *buf) {
    QFileInfo info(texmacs_utf8_string_to_system_string(filename));
    
    if (!info.exists()) {
        return -1; // File does not exist
    }
    
    buf->st_mode = (info.isDir() ? S_IFDIR : S_IFREG) 
                 | (info.isReadable() ? S_IRUSR : 0) 
                 | (info.isWritable() ? S_IWUSR : 0) 
                 | (info.isExecutable() ? S_IXUSR : 0);
    buf->st_size = info.size();
#if QT_VERSION >= 0x060000
    buf->st_mtime = info.lastModified().toSecsSinceEpoch();
    buf->st_atime = info.lastRead().toSecsSinceEpoch();
    buf->st_ctime = info.birthTime().toSecsSinceEpoch();
#else
    buf->st_mtime = info.lastModified().toTime_t();
    buf->st_atime = info.lastRead().toTime_t();
    buf->st_ctime = info.created().toTime_t();
#endif
    return 0; // Success
}

bool texmacs_mkdir(string dirname, int mode) {
  return QDir().mkdir(texmacs_utf8_string_to_system_string(dirname));
}

bool texmacs_rmdir(string dirname) {
  return QDir().rmdir(texmacs_utf8_string_to_system_string(dirname));
}

bool texmacs_rename(string oldname, string newname) {
  return QFile::rename(
    texmacs_utf8_string_to_system_string(oldname),
    texmacs_utf8_string_to_system_string(newname)
  );
}

bool texmacs_chmod(string filename, int mode) {
  // do nothing
}

bool texmacs_remove(string filename) {
  return QFile::remove(texmacs_utf8_string_to_system_string(filename));
}

bool texmacs_getenv(string variable_name, string &variable_value) {
  QByteArray value = qgetenv(texmacs_utf8_string_to_system_string(variable_name).toUtf8());
  if (value.isEmpty()) {
    return false;
  }
  variable_value = texmacs_ainsi_to_utf8(value);
  return true;
}

bool texmacs_setenv(string variable_name, string new_value) {
  return qputenv(
    texmacs_utf8_string_to_system_string(variable_name).toUtf8(),
    texmacs_utf8_string_to_system_string(new_value).toUtf8()
  );
}

string get_default_theme() {
#if QT_VERSION >= 0x060000
  if (QGuiApplication::styleHints()->colorScheme() == Qt::ColorScheme::Dark) {
    return "dark";
  } else {
    return "light";
  }
#else
  return "light";
#endif
}

int texmacs_guile_printf(const char *format, ...) {
  va_list args;
  va_start(args, format);
  QString output = QString::vasprintf(format, args);
  va_end(args);
  qDebug() << output;
  return output.size();
}

int texmacs_guile_fprintf(FILE *stream, const char *format, ...) {
  if (stream != stdout && stream != stderr) {
    va_list args;
    va_start(args, format);
    vfprintf(stream, format, args);
    va_end(args);
    return 0;
  }

  va_list args;
  va_start(args, format);
  QString output = QString::vasprintf(format, args);
  va_end(args);
  
  if (stream == stdout) {
    qDebug() << output;
  } else if (stream == stderr) {
    qWarning() << output;
  }
  
  return output.size();
}

void texmacs_init_guile_hooks() {
  guile_fprintf = texmacs_guile_fprintf;
  guile_printf = texmacs_guile_printf;
}